package org.rfdump;

/**
 *
 * RF Dump V1.2
 * @author Boris Wolf, Lukas Grunwald
 * @version 07/12/2004
 *
 */

import java.util.Properties;
import java.io.FileInputStream;

public class RFID {

    public Properties props = new Properties();
    public String portName = "";

    public static final int WAIT = 500;

    public static final String CMD_RESET = "x";
    public static final String CMD_VERSION = "v";
    public static final String CMD_SELECT = "s";
    public static final String CMD_READ_PAGE = "r";
    public static final String CMD_WRITE_PAGE = "w";

    public static final char ERR_UNKNOWN_CMD = '?';
    public static final char ERR_GENERAL_FAILURE = 'F';
    public static final char ERR_INVALID_DATA = 'I';
    public static final char ERR_NO_TAG = 'N';
    public static final char ERR_WRITE_FAILURE = 'U';
    public static final char ERR_PAGE_LOCKED = 'L';

    public static final String MSG_UNKNOWN_CMD = "Unknown Command";
    public static final String MSG_GENERAL_FAILURE = "General Failure";
    public static final String MSG_INVALID_DATA = "Invalid Data";
    public static final String MSG_NO_TAG = "No Tag";
    public static final String MSG_WRITE_FAILURE = "Write Failure";
    public static final String MSG_PAGE_LOCKED = "Lock Failure";

    public static final char TAG_TYPE_ISO15693 = 'V';
    public static final char TAG_TYPE_TAGIT = 'T';
    public static final char TAG_TYPE_ICODE = 'I';
    public static final char TAG_TYPE_MIFARE = 'M';
    public static final char TAG_TYPE_SR176 = 'S';
    public static final char TAG_TYPE_ISO1443B = 'Z';

    public static final String TAG_NAME_ISO15693 = "ISO 15693";
    public static final String TAG_NAME_TAGIT = "Tagit";
    public static final String TAG_NAME_ICODE = "Icode";
    public static final String TAG_NAME_MIFARE = "Mifare (ISO 14443 Type A)";
    public static final String TAG_NAME_SR176 = "SR176";
    public static final String TAG_NAME_ISO1443B = "ISO 14443 Type B";

    public static final String TAG_MFR_STMICROELECTRONICS = "ST Microelectronics";
    public static final String TAG_MFR_PHILLIPS = "Phillips Semiconductors";
    public static final String TAG_MFR_INFINEON = "Infinion Technologies AG";
    public static final String TAG_MFR_TEXASINSTRUMENTS = "Texas Instruments";
    public static final String TAG_MFR_EMMICROELECTRONICS = "EM Microelectronic-Marin SA";

    private SerialConnection sc;

    public String currentTagID;

    public static void main(String[] args) {

        RFID rfid = new RFID();

        rfid.reset();

        String version = rfid.getVersion();
        System.out.println("VERSION:\t" + version);

        while (true) {

            try {Thread.sleep(100);} catch (Exception e) {}

            String select = rfid.select();

            System.out.println("SELECT:\t" + select);

            String errMsg = rfid.getErrorMsg(select);

            if (errMsg != null) {

                System.out.println("ERROR:\t" + errMsg);

            } else {

                // read tag info

                String tagType = rfid.getTagTypeName(select);
                System.out.println("TYPE:\t" + tagType);

                String tagID = rfid.getTagID(select);
                System.out.println("TAGID:\t" + tagID);

                if (!tagID.equals(rfid.currentTagID)) {

                    System.out.println("NEW TAG FOUND: READING MEMORY");
                    rfid.currentTagID = tagID;

                    // read tag memory

                    int adr = 0;

                    while (errMsg == null) {

                        String hexAdr = Integer.toHexString(adr);

                        // only two-digit adresses are possible

                        if (hexAdr.equals("100"))
                            break;

                        if (hexAdr.length() == 1)
                            hexAdr = "0" + hexAdr;

                        String page = rfid.readPage(hexAdr);

                        errMsg = rfid.getErrorMsg(page);

                        if (errMsg != null) {
                            System.out.println(hexAdr + ":\t" + errMsg);
                        }   else {
                            System.out.println(hexAdr + ":\t" + page);
                        }

                        adr++;
                    }
                }
            }
        }
    }

    public RFID() {

        try {

            FileInputStream in = new FileInputStream("rfdump.properties");
            this.props.load(in);
            this.portName = this.props.getProperty("port");
            in.close();

        } catch (Exception e) {
            System.err.println("Error reading properties file rfdump.properties.");
        }

        sc = new SerialConnection(this.portName);
        currentTagID = "";
    }

    public void close() {
        sc.close();
    }

    public String getTagID(String select) {

        String ID = null;

        if (select != null) {
            if (select.length() > 1) {
                ID = select.substring(1);
            }
        }

        return ID;
    }

    public char getTagType(String select) {

        char tagType = ' ';

        if (select != null) {
            if (select.length() >= 1) {
                tagType = select.charAt(0);
            }
        }

        return tagType;
    }

    public String getTagTypeName(String select) {

        String tagType = null;

        if (select != null) {
            if (select.length() >= 1) {

                switch (select.charAt(0)) {

                    case TAG_TYPE_ISO15693 :
                        tagType = TAG_NAME_ISO15693;
                        break;
                    case TAG_TYPE_TAGIT :
                        tagType = TAG_NAME_TAGIT;
                        break;
                    case TAG_TYPE_ICODE :
                        tagType = TAG_NAME_ICODE;
                        break;
                    case TAG_TYPE_MIFARE :
                        tagType = TAG_NAME_MIFARE;
                        break;
                    case TAG_TYPE_SR176 :
                        tagType = TAG_NAME_SR176;
                        break;
                    case TAG_TYPE_ISO1443B :
                        tagType = TAG_NAME_ISO1443B;
                        break;
                }
            }
        }

        return tagType;
    }

    public String getTagManufacturerName(String select) {

        String tagManufacturer = null;

        if (select != null) {
            if (select.length() >= 1) {

                char tagType = getTagType(select);

                if (tagType == RFID.TAG_TYPE_ISO15693) {

                    String mfidStr = select.substring(3,5);
                    int mfid = (new Integer(mfidStr)).intValue();

                    if (mfid == 0x02)
                        tagManufacturer = RFID.TAG_MFR_STMICROELECTRONICS;
                    else if (mfid == 0x04)
                        tagManufacturer = RFID.TAG_MFR_PHILLIPS;
                    else if (mfid == 0x05)
                        tagManufacturer = RFID.TAG_MFR_INFINEON;
                    else if (mfid == 0x07)
                        tagManufacturer = RFID.TAG_MFR_TEXASINSTRUMENTS;
                    else if (mfid == 0x16)
                        tagManufacturer = RFID.TAG_MFR_EMMICROELECTRONICS;

                } else if (tagType == RFID.TAG_TYPE_TAGIT) {
                    tagManufacturer = RFID.TAG_MFR_TEXASINSTRUMENTS;
                } else if (tagType == RFID.TAG_TYPE_ICODE) {
                    tagManufacturer = RFID.TAG_MFR_PHILLIPS;
                } else if (tagType == RFID.TAG_TYPE_SR176) {
                    tagManufacturer = RFID.TAG_MFR_STMICROELECTRONICS;
                }
            }
        }

        return tagManufacturer;
    }

    public String getErrorMsg(String errCode) {

        String errMsg = null;

        if (errCode != null) {
            if (errCode.length() == 1) {

                switch (errCode.charAt(0)) {

                    case ERR_UNKNOWN_CMD :
                        errMsg = MSG_UNKNOWN_CMD;
                        break;
                    case ERR_GENERAL_FAILURE :
                        errMsg = MSG_GENERAL_FAILURE;
                        break;
                    case ERR_INVALID_DATA :
                        errMsg = MSG_INVALID_DATA;
                        break;
                    case ERR_NO_TAG :
                        errMsg = MSG_NO_TAG;
                        break;
                    case ERR_WRITE_FAILURE :
                        errMsg = MSG_WRITE_FAILURE;
                        break;
                    case ERR_PAGE_LOCKED :
                        errMsg = MSG_PAGE_LOCKED;
                        break;
                }
            }
        }

        return errMsg;
    }

    public void reset() {
        try {Thread.sleep(WAIT);} catch (Exception e) {}
        sc.write(CMD_RESET);
        //sc.write(CMD_SELECT);
        try {Thread.sleep(WAIT);} catch (Exception e) {}
        sc.resetLineBuffer();
        try {Thread.sleep(WAIT);} catch (Exception e) {}
    }

    public String getVersion() {

        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        sc.write(CMD_VERSION);
        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        String response = sc.readNextLine();
        //try {Thread.sleep(WAIT);} catch (Exception e) {}

        return response;
    }

    public String select() {

        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        sc.write(CMD_SELECT);
        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        String response = sc.readNextLine();

        if (response == null)
            response = sc.readNextLine();
        else if (response.equals(""))
            response = sc.readNextLine();

        //try {Thread.sleep(WAIT);} catch (Exception e) {}

        return response;
    }

    public String readPage(int pageNoInt) {

        String pageNo = Integer.toHexString(pageNoInt);

        if (pageNo.length() == 1)
            pageNo = "0" + pageNo;

        return readPage(pageNo);
    }

    public String readPage(String pageNo) {

        // pageNo needs to be in hex representation

        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        sc.write(CMD_READ_PAGE + pageNo);
        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        String page = sc.readNextLine();
        //try {Thread.sleep(WAIT);} catch (Exception e) {}

        if (page == null)
            page = sc.readNextLine();
        else if (page.equals(""))
            page = sc.readNextLine();

        return page;
    }

    public String writePage(String pageNo, String page) {

        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        sc.write(CMD_WRITE_PAGE + pageNo + page);
        //try {Thread.sleep(WAIT);} catch (Exception e) {}
        String response = sc.readNextLine();
        //try {Thread.sleep(WAIT);} catch (Exception e) {}

        if (response == null)
            response = sc.readNextLine();
        else if (response.equals(""))
            response = sc.readNextLine();

        return response;
    }
}
