package org.rfdump;

/**
 *
 * RF Dump V1.2
 * @author Boris Wolf, Lukas Grunwald
 * @version 07/12/2004
 *
 */

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;

import java.awt.*;

import java.io.Serializable;

public class RFIDTableCellRenderer extends DefaultTableCellRenderer implements TableCellRenderer, Serializable {

    protected static Border noFocusBorder = new EmptyBorder(1, 1, 1, 1);

    private Color unselectedForeground;
    private Color unselectedBackground;

    public RFIDTableCellRenderer() {
        super();
        setOpaque(true);
        setBorder(noFocusBorder);
    }

    public void setForeground(Color c) {
        super.setForeground(c);
        unselectedForeground = c;
    }

    public void setBackground(Color c) {
        super.setBackground(c);
        unselectedBackground = c;
    }

    public void updateUI() {
        super.updateUI();
        setForeground(null);
        setBackground(null);
    }

    public Component getTableCellRendererComponent(JTable table, Object value,
                          boolean isSelected, boolean hasFocus, int row, int column) {

        if (isSelected) {
            super.setForeground(table.getSelectionForeground());
            super.setBackground(table.getSelectionBackground());
        } else {
            super.setForeground((unselectedForeground != null) ? unselectedForeground : table.getForeground());
            super.setBackground((unselectedBackground != null) ? unselectedBackground : table.getBackground());
        }

        setFont(new Font("Monospaced", Font.PLAIN, 12));

        if (hasFocus) {

            setBorder(UIManager.getBorder("Table.focusCellHighlightBorder"));

            if (table.isCellEditable(row, column)) {
                super.setForeground(UIManager.getColor("Table.focusCellForeground"));
                super.setBackground(UIManager.getColor("Table.focusCellBackground"));
            }

        } else {
            setBorder(noFocusBorder);
        }

        setValue(value);

        return this;
    }


    protected void setValue(Object value) {
    	setText((value == null) ? "" : value.toString());
    }
}
