package org.rfdump;

/**
 *
 * RF Dump V1.2
 * @author Boris Wolf, Lukas Grunwald
 * @version 07/12/2004
 *
 */

import javax.swing.table.*;

import java.util.ArrayList;
import java.util.HashMap;

public class RFIDTableModel extends AbstractTableModel {

    private ArrayList memory = new ArrayList();
    private ArrayList writeBuffer;

    private static final String BLANK_PAGE = "--------";

	public RFIDTableModel(ArrayList memory, ArrayList writeBuffer) {

        this.memory = memory;
        this.writeBuffer = writeBuffer;
    }

    public void append(String value) {

        this.memory.add(value);

        int column = (this.memory.size()-1)%8+1;
        int row = (this.memory.size()-((this.memory.size()-1)%8))/8;

        this.fireTableCellUpdated(row, column);
    }

//    public void setPage(int adr, String value) {
//
//        this.memory.set(adr, value);
//
//        int column = adr%8+1;
//        int row = ((adr-1)-((this.memory.size()-1)%8))/8;
//
//        this.fireTableCellUpdated(row, column);
//    }

    public String getColumnName(int col) {

        String header = "";

        switch (col) {

            case 0:
                header = "Adr";
                break;
            case 1:
                header = "0 / 8";
                break;
            case 2:
                header = "1 / 9";
                break;
            case 3:
                header = "2 / A";
                break;
            case 4:
                header = "3 / B";
                break;
            case 5:
                header = "4 / C";
                break;
            case 6:
                header = "5 / D";
                break;
            case 7:
                header = "6 / E";
                break;
            case 8:
                header = "7 / F";
                break;
            case 9:
                header = "ASCII";
                break;
        }

        return header;
    }

    public int getRowCount() { 
        return 32;
	}

    public int getColumnCount() { 
		return 10;
	}

    public Object getValueAt(int row, int col) {

        String value = "";

        if (col == 0) {

            // address column

            if (row%2==1)
                row--;

            row /= 2;

            value = Integer.toHexString(row).toUpperCase();

        } else if (col == 9) {

            // ASCII column

            StringBuffer hexRow = new StringBuffer("");

            for (int i=1; i<=8; i++) {

                int idx = row*8 + (i-1);

                if (idx <= memory.size()-1)
                    hexRow.append((String)memory.get(idx));
                else
                    hexRow.append(BLANK_PAGE);
            }

            StringBuffer ascii = new StringBuffer("");

            for (int i=0; i<hexRow.length()/2; i++) {

                String byteString = hexRow.substring(i*2,i*2+2);

                char c = '-';

                if (!byteString.equals("--"))
                    c = (char)(Integer.valueOf(byteString,16).intValue());

                if ((int)c < 0x20)
                    c = '.';

                ascii.append(c);
            }

            value = ascii.toString();

        } else {

            // hex column

            int idx = row*8 + (col-1);

            if (idx <= memory.size()-1) {
                value = (String)memory.get(idx);
            } else {
                value = BLANK_PAGE;
            }
        }

        return value;
    }

    public boolean isCellEditable(int row, int col) {

        if (col>0)
    		return true;
        else
            return false;
	}

    public void setValueAt(Object value, int adr) {

        int column = adr%8+1;
        int row = (adr-adr%8)/8;

        setValueAt(value, row, column);

        this.fireTableCellUpdated(row, column);
    }

    public void setValueAt(Object value, int row, int col) {

        String strValue = (String)value;

        int PAGE_LENGTH = 16;

        if (this.memory.size()>0)
            PAGE_LENGTH = ((String)this.memory.get(0)).length();

        if (col == 9) {

            // ASCII editor

            int ROW_LENGTH = 8*PAGE_LENGTH/2;

            if (strValue.length() > ROW_LENGTH) {

                strValue = strValue.substring(0, ROW_LENGTH);

            } else if (strValue.length() < ROW_LENGTH) {

                String padding = "";

                for (int i=0; i<ROW_LENGTH-strValue.length(); i++)
                    padding += " ";

                strValue = strValue + padding;
            }

            String hexPage = "";
            int pageColumnIndex = 1;

            for (int i=0; i<ROW_LENGTH; i++) {

                String hexNibble = Integer.toHexString((int)strValue.charAt(i));

                if (hexNibble.length() == 1)
                    hexNibble = "0" + hexNibble;

                hexPage += hexNibble;

                if (hexPage.length() == PAGE_LENGTH) {
                    setValueAt(hexPage, row, pageColumnIndex);
                    pageColumnIndex++;
                    hexPage = "";
                }
            }

        } else {

            // HEX editor

            if (strValue.length() > PAGE_LENGTH) {

                strValue = strValue.substring(0, PAGE_LENGTH);

            } else if (strValue.length() < PAGE_LENGTH) {

                String padding = "";

                for (int i=0; i<PAGE_LENGTH-strValue.length(); i++)
                    padding += "0";

                strValue = padding + strValue;
            }

            int adr = row*8 + (col-1);
            String hexAdr = Integer.toHexString(adr);

            if (hexAdr.length() == 1)
                hexAdr = "0" + hexAdr;

            memory.set(adr, strValue);
            writeBuffer.add(hexAdr + strValue);
            this.fireTableCellUpdated(row, 9);
        }
    }
}

