package org.rfdump;

/**
 *
 * RF Dump V1.2
 * @author Boris Wolf, Lukas Grunwald
 * @version 07/12/2004
 *
 */

import java.io.*;
import java.util.*;
import javax.comm.*;

public class SerialConnection implements Runnable, SerialPortEventListener {

    static CommPortIdentifier portId;

    InputStream inputStream;
    static OutputStream outputStream;
    SerialPort serialPort;
    Thread readThread;

    StringBuffer inputBuffer;
    ArrayList pendingLines;

    public static void main(String[] args) {
    }

    public SerialConnection(String portName) {

        System.out.print("Trying to open port " + portName + "...");
        boolean failed = false;

        inputBuffer = new StringBuffer();
        pendingLines = new ArrayList();

        try {
            portId = CommPortIdentifier.getPortIdentifier(portName);
        } catch (NoSuchPortException e) {
            System.err.println("Error: " + e);
            failed = true;
        }

        if (!failed) {
            try {
                serialPort = (SerialPort)portId.open("RFDump", 2000);
            } catch (PortInUseException e) {
                System.err.println("Error: " + e);
                failed = true;
            }
        }

        if (!failed) {
            try {
                inputStream = serialPort.getInputStream();
                outputStream = serialPort.getOutputStream();
            } catch (IOException e) {
                System.err.println("Error: " + e);
                failed = true;
            }
        }

        if (! failed) {
            try {
                serialPort.addEventListener(this);
                serialPort.notifyOnDataAvailable(true);
            } catch (TooManyListenersException e) {
                System.err.println("Error: " + e);
                failed = true;
            }
        }

        if (! failed) {
            try {

                serialPort.setSerialPortParams(9600,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);

            } catch (UnsupportedCommOperationException e) {
                System.err.println("Error: " + e);
                failed = true;
            }
        }

        if (failed) {
            System.out.println("failed");
        } else {
            System.out.println("ok");
            readThread = new Thread(this);
            readThread.start();
        }
    }

    public void close() {

        try {
            inputStream.close();
            outputStream.close();
        } catch (Exception e) {
            System.err.println("Error: " + e);
        }

        serialPort.close();
    }

    public void write(String msg) {

        try {
            outputStream.write(msg.getBytes());
        } catch(java.io.IOException e) {
            System.err.println("Error: " + e);
        }
    }

    public String readNextLine() {
        while (pendingLines.size() == 0) {}
        String line = (String)pendingLines.remove(0);
        return line;
    }

    public void resetLineBuffer() {
        pendingLines = new ArrayList();
    }

    public void run() {
        try {
            Thread.sleep(20000);
        } catch (InterruptedException e) {}
    }

    public void serialEvent(SerialPortEvent event) {

        switch(event.getEventType()) {

            case SerialPortEvent.BI:
            case SerialPortEvent.OE:
            case SerialPortEvent.FE:
            case SerialPortEvent.PE:
            case SerialPortEvent.CD:
            case SerialPortEvent.CTS:
            case SerialPortEvent.DSR:
            case SerialPortEvent.RI:
            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:

                break;

            case SerialPortEvent.DATA_AVAILABLE:

                int newData = 0;

                while (newData != -1) {

                    try {

                        newData = inputStream.read();

                        if (newData == -1) {
                            break;
                        }

                        if ('\r' == (char)newData || '\n' == (char)newData) {

                            pendingLines.add(new String(inputBuffer.toString()));
                            inputBuffer = new StringBuffer();

                        } else {
                            inputBuffer.append((char)newData);
                        }

                    } catch (IOException e) {
                        System.err.println("Error: " + e);
                        return;
                    }
                }
        }
    }
}
